"""Python class to communicate with Metron.cloud"""
import json
import platform
import ssl
import time
from datetime import datetime
from typing import Dict, Optional, Tuple
from urllib.error import HTTPError, URLError
from urllib.request import Request, urlopen

from darkseid.genericmetadata import GenericMetadata
from darkseid.issuestring import IssueString
from darkseid.utils import list_to_string
from ratelimit import limits, sleep_and_retry

from .. import VERSION

ONE_MINUTE = 60


class MetronTalker:
    """Python class to communicate with Metron's REST"""

    def __init__(self, auth: bytes) -> None:
        self.api_base_url = "https://metron.cloud/api"
        self.auth_str = f"Basic {auth.decode('utf-8')}"
        self.user_agent = (
            f"Metron-Tagger/{VERSION} ({platform.system()}; {platform.release()})"
        )
        self.ssl = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)

    @classmethod
    def parse_date_string(
        cls, date_str: str
    ) -> Tuple[Optional[str], Optional[str], Optional[str]]:
        """
        Classmethod that takes a date from Metron's REST API and splits it into it's components
        """
        day = None
        month = None
        year = None
        if date_str is not None:
            parts = date_str.split("-")
            year = parts[0]
            if len(parts) > 1:
                month = parts[1]
                if len(parts) > 2:
                    day = parts[2]
        return day, month, year

    @sleep_and_retry
    @limits(calls=20, period=ONE_MINUTE)
    def fetch_response(self, url: str):
        """Function to retrieve a response from Metron's REST API"""
        request = Request(url)
        request.add_header("Authorization", self.auth_str)
        request.add_header("User-Agent", self.user_agent)

        try:
            content = urlopen(request, context=self.ssl)
        except HTTPError as http_error:
            # TODO: Look into handling throttling better, but for now let's use this.
            if http_error.code == 429:
                print("Exceeded api rate limit. Sleeping for 30 seconds...")
                time.sleep(30)
                return self.fetch_response(url)
            print(f"HTTP error: {http_error.reason}")
        except URLError as url_error:
            print(f"Connection error: {url_error.reason}")
        else:
            return json.loads(content.read().decode("utf-8"))

    def fetch_issue_data_by_issue_id(self, issue_id: int) -> GenericMetadata:
        """Method to get an issue's metadata by supplying the issue id"""
        url = f"{self.api_base_url}/issue/{issue_id}/?format=json"
        resp = self.fetch_response(url)
        meta_data = self.map_metron_data_to_metadata(resp)
        meta_data.is_empty = False

        return meta_data

    def search_for_issue(self, query_dict: Dict[str, str]):
        """
        Method to search for an issue based on a dictionary of
        words, volume number, or year.
        """
        url = f"{self.api_base_url}/issue/?series_name={query_dict['series']}&number={query_dict['number']}"
        if query_dict["volume"]:
            url += f"&series_volume={query_dict['volume']}"
        if query_dict["year"]:
            url += f"&cover_year={query_dict['year']}"
        url += "&format=json"
        return self.fetch_response(url)

    def map_metron_data_to_metadata(self, issue_results) -> GenericMetadata:
        """
        Method to map the issue results from Metron's REST API
        to metadata
        """
        metadata = GenericMetadata()

        metadata.series = issue_results["series"]["name"]
        metadata.volume = issue_results["volume"]

        metadata.issue = IssueString(issue_results["number"]).as_string()

        titles = issue_results["name"]
        if titles:
            title_list = [title for title in titles]
            metadata.title = list_to_string(title_list)

        metadata.publisher = issue_results["publisher"]["name"]
        metadata.day, metadata.month, metadata.year = self.parse_date_string(
            issue_results["cover_date"]
        )

        metadata.comments = issue_results["desc"]

        now_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        metadata.notes = (
            f"Tagged with MetronTagger-{VERSION} using info from Metron on {now_date}. "
            + f"[issue_id:{issue_results['id']}]"
        )

        person_credits = issue_results["credits"]
        if person_credits:
            for person in person_credits:
                if "role" in person:
                    roles = person["role"]
                    for role in roles:
                        metadata.add_credit(person["creator"], role["name"])

        character_credits = issue_results["characters"]
        if character_credits:
            character_list = [character["name"] for character in character_credits]
            metadata.characters = list_to_string(character_list)

        team_credits = issue_results["teams"]
        if team_credits:
            team_list = [team["name"] for team in team_credits]
            metadata.teams = list_to_string(team_list)

        story_arc_credits = issue_results["arcs"]
        if story_arc_credits:
            arc_list = [arc["name"] for arc in story_arc_credits]
            metadata.story_arc = list_to_string(arc_list)

        return metadata



"""Main project file"""
from argparse import Namespace
from base64 import standard_b64encode
from pathlib import Path
from sys import exit
from typing import List, Optional, Tuple

from darkseid.comicarchive import ComicArchive
from darkseid.genericmetadata import GenericMetadata
from darkseid.utils import get_recursive_filelist

from .taggerlib.filerenamer import FileRenamer
from .taggerlib.filesorter import FileSorter
from .taggerlib.metrontalker import MetronTalker
from .taggerlib.options import make_parser
from .taggerlib.settings import MetronTaggerSettings
from .taggerlib.utils import create_issue_query_dict

# Load the settings
SETTINGS = MetronTaggerSettings()


# TODO: Consider making this a dict or tuple
class MultipleMatch:
    """Class to hold information on searches with multiple matches"""

    def __init__(self, filename, match_list) -> None:
        self.filename = filename
        self.matches = match_list


# TODO: Consider making this a dict or tuple
class OnlineMatchResults:
    """Class to track online match results"""

    def __init__(self) -> None:
        self.good_matches: List[str] = []
        self.no_matches: List[str] = []
        self.multiple_matches: List[str] = []


def create_metron_talker() -> MetronTalker:
    """Function that creates the metron talker"""
    auth = f"{SETTINGS.metron_user}:{SETTINGS.metron_pass}"
    base64string = standard_b64encode(auth.encode("utf-8"))
    return MetronTalker(base64string)


def create_pagelist_metadata(comic_archive: ComicArchive) -> GenericMetadata:
    """Function that returns the metadata for the total number of pages"""
    meta_data = GenericMetadata()
    meta_data.set_default_page_list(comic_archive.get_number_of_pages())

    return meta_data


def write_issue_metadata(filename: Path, issue_id: int, talker: MetronTalker) -> None:
    """
    Function to get an issue's metadata from Metron and the write that
    information to a tag in the comic archive
    """
    success = False

    metron_md = talker.fetch_issue_data_by_issue_id(issue_id)
    if metron_md:
        comic_archive = ComicArchive(filename)
        meta_data = create_pagelist_metadata(comic_archive)
        metron_md.overlay(meta_data)
        success = comic_archive.write_metadata(metron_md)

    if success:
        print(f"match found for '{filename.name}'")
    else:
        print(f"there was a problem writing metadate for '{filename.name}'")


def select_choice_from_multiple_matches(filename: Path, match_set) -> Optional[int]:
    """
    Function to ask user to choice which issue metadata to write,
    when there are multiple choices
    """
    print(f"\n{filename.name} - Multiple results found:")

    # sort match list by cover date
    match_set = sorted(match_set, key=lambda m: m["cover_date"])

    for (counter, match) in enumerate(match_set, start=1):
        print(f"{counter}. {match['__str__']} ({match['cover_date']})")

    while True:
        i = input("Choose a match #, or 's' to skip: ")
        if (i.isdigit() and int(i) in range(1, len(match_set) + 1)) or i == "s":
            break

    if i != "s":
        i = int(i) - 1
        issue_id = match_set[i]["id"]
    else:
        issue_id = None

    return issue_id


def workable_comic(comic: ComicArchive) -> bool:
    return comic.is_writable() and comic.seems_to_be_a_comic_archive()


def process_file(
    filename: Path, match_results, talker: MetronTalker
) -> Tuple[Optional[int], Optional[bool]]:
    """
    Main function to attempt query Metron and write a tag
    """
    comic_archive = ComicArchive(filename)

    if not workable_comic(comic_archive):
        print(f"{filename.name} may not be a comic or writable")
        return None, False

    query_dict = create_issue_query_dict(filename)
    res = talker.search_for_issue(query_dict)
    res_count = res["count"]

    issue_id = None
    multiple_match = False
    if not res_count > 0:
        issue_id = None
        match_results.no_matches.append(filename)
        multiple_match = False
    elif res_count > 1:
        issue_id = None
        match_results.multiple_matches.append(MultipleMatch(filename, res["results"]))
        multiple_match = True
    elif res_count == 1:
        issue_id = res["results"][0]["id"]
        match_results.good_matches.append(filename)
        multiple_match = False

    return issue_id, multiple_match


def post_process_matches(match_results, talker: MetronTalker) -> None:
    """
    Function to print match results and if there are
    multiple matches, prompt the user
    """
    # Print file matching results.
    if match_results.good_matches:
        print("\nSuccessful matches:\n------------------")
        for comic in match_results.good_matches:
            print(comic)

    if match_results.no_matches:
        print("\nNo matches:\n------------------")
        for comic in match_results.no_matches:
            print(comic)

    # Handle files with multiple matches.
    if match_results.multiple_matches:
        for match_set in match_results.multiple_matches:
            issue_id = select_choice_from_multiple_matches(
                match_set.filename, match_set.matches
            )
            if issue_id:
                write_issue_metadata(match_set.filename, issue_id, talker)


def list_comics_with_missing_metadata(file_list: List[Path]) -> None:
    print("\nShowing files without metadata:\n-------------------------------")
    for comic in file_list:
        comic_archive = ComicArchive(comic)
        if comic_archive.has_metadata():
            continue
        print(f"no metadata in '{comic.name}'")


def delete_comics_metadata(file_list: List[Path]) -> None:
    print("\nRemoving metadata:\n-----------------")
    for comic in file_list:
        comic_archive = ComicArchive(comic)
        if comic_archive.has_metadata():
            comic_archive.remove_metadata()
            print(f"removed metadata from '{comic.name}'")
        else:
            print(f"no metadata in '{comic.name}'")


def sort_list_of_comics(file_list: List[Path]) -> None:
    if not SETTINGS.sort_dir:
        print("\nUnable to sort files. No destination directory was provided.")
        return

    print("\nStarting sorting of comic archives:\n----------------------------------")
    file_sorter = FileSorter(SETTINGS.sort_dir)
    for comic in file_list:
        result = file_sorter.sort_comics(comic)
        if not result:
            print(f"unable to move {comic.name}.")


def retrieve_single_issue_from_id(file_list: List[Path], id: int) -> None:
    if len(file_list) > 1:
        print("More than one file was passed for Id processing. Exiting...")
        exit(0)

    filename = file_list[0]
    talker = create_metron_talker()
    write_issue_metadata(filename, id, talker)


def get_options() -> Namespace:
    parser = make_parser()
    opts = parser.parse_args()

    if opts.user:
        SETTINGS.metron_user = opts.user

    if opts.password:
        SETTINGS.metron_pass = opts.password

    if opts.sort_dir:
        SETTINGS.sort_dir = opts.sort_dir

    if opts.set_metron_user or opts.set_sort_dir:
        SETTINGS.save()

    return opts


def identify_comics(file_list: List[Path], ignore: bool) -> None:
    print("\nStarting online search and tagging:\n----------------------------------")

    # Initialize class to handle results for files with multiple matches
    match_results = OnlineMatchResults()
    talker = create_metron_talker()

    # Let's look online to see if we can find any matches on Metron.
    for filename in file_list:
        if ignore:
            comic_archive = ComicArchive(filename)
            if comic_archive.has_metadata():
                print(f"{filename.name} has metadata. Skipping...")
                continue

        issue_id, multiple_match = process_file(filename, match_results, talker)
        if issue_id:
            write_issue_metadata(filename, issue_id, talker)
        else:
            if not multiple_match:
                print(f"no match for '{filename.name}'")
                continue

    # Print match results & handle files with multiple matches
    post_process_matches(match_results, talker)


def rename_comics(file_list: List[Path]) -> List[Path]:
    print("\nStarting comic archive renaming:\n-------------------------------")

    # Lists to track filename changes
    new_file_names: List[Path] = []
    original_files_changed: List[Path] = []
    for comic in file_list:
        comic_archive = ComicArchive(comic)
        if not comic_archive.has_metadata():
            print(f"skipping '{comic.name}'. no metadata available.")
            continue

        meta_data = comic_archive.read_metadata()
        renamer = FileRenamer(meta_data)
        unique_name = renamer.rename_file(comic)
        if unique_name is None:
            continue

        # track what files are being renamed
        new_file_names.append(unique_name)
        original_files_changed.append(comic)

        print(f"renamed '{comic.name}' -> '{unique_name.name}'")

    # Update file_list for renamed files
    for original_file in original_files_changed:
        file_list.remove(original_file)

    for new_file in new_file_names:
        file_list.append(new_file)

    return file_list


def main() -> None:
    """
    Main func
    """
    opts = get_options()

    # Parse paths to get file list
    file_list = []
    file_list = get_recursive_filelist(opts.path)

    if not file_list:
        print("No files to process. Exiting.")
        exit(0)

    if opts.missing:
        list_comics_with_missing_metadata(file_list)

    if opts.delete:
        delete_comics_metadata(file_list)

    if opts.id:
        retrieve_single_issue_from_id(file_list, opts.id)

    if opts.online:
        identify_comics(file_list, opts.ignore_existing)

    if opts.rename:
        file_list = rename_comics(file_list)

    if opts.sort:
        sort_list_of_comics(file_list)


if __name__ == "__main__":
    exit(main())



"""Class to handle project settings"""
import configparser
import platform
from os import environ
from pathlib import Path, PurePath
from typing import Optional


class MetronTaggerSettings:
    """Class to handle project settings"""

    @staticmethod
    def get_settings_folder() -> Path:
        """Method to determine where the users settings should be saved"""

        if platform.system() == "Windows":
            windows_path = PurePath(environ["APPDATA"]).joinpath("MetronTagger")
            folder = Path(windows_path)
        else:
            folder = Path.home() / ".MetronTagger"

        return folder

    def __init__(self, config_dir: Optional[str] = None) -> None:
        # Metron creditials
        self.metron_user: str = ""
        self.metron_pass: str = ""
        self.sort_dir: str = ""

        self.config = configparser.ConfigParser()

        if not config_dir:
            self.folder = MetronTaggerSettings.get_settings_folder()
        else:
            self.folder = Path(config_dir)

        self.settings_file = self.folder / "settings.ini"

        if not self.settings_file.parent.exists():
            self.settings_file.parent.mkdir()

        # Write the config file if it doesn't exist
        if not self.settings_file.exists():
            self.save()
        else:
            self.load()

    def load(self) -> None:
        """Method to retrieve a users settings"""
        self.config.read(self.settings_file)

        if self.config.has_option("metron", "user"):
            self.metron_user = self.config["metron"]["user"]

        if self.config.has_option("metron", "password"):
            self.metron_pass = self.config["metron"]["password"]

        if self.config.has_option("DEFAULT", "sort_dir"):
            self.sort_dir = self.config["DEFAULT"]["sort_dir"]

    def save(self) -> None:
        """Method to save a users settings"""
        if not self.config.has_section("metron"):
            self.config.add_section("metron")

        self.config["DEFAULT"]["sort_dir"] = self.sort_dir
        self.config["metron"]["user"] = self.metron_user
        self.config["metron"]["password"] = self.metron_pass

        with self.settings_file.open("w") as configfile:
            self.config.write(configfile)



"""Class to sort comic file based on it's metadata tags"""
import pathlib
from os import fspath
from pathlib import Path
from shutil import Error, move

from darkseid.comicarchive import ComicArchive

from .utils import cleanup_string


class FileSorter:
    """Class to move comic files based on it's metadata tags"""

    def __init__(self, directory: str) -> None:
        self.sort_directory = directory

    def sort_comics(self, comic: Path) -> bool:
        """Method to move the comic file based on it's metadata tag"""
        comic_archive = ComicArchive(comic)
        if comic_archive.has_metadata():
            meta_data = comic_archive.read_metadata()
        else:
            return False

        publisher = cleanup_string(meta_data.publisher)
        series = cleanup_string(meta_data.series)
        volume = cleanup_string(meta_data.volume)
        if volume is not None:
            volume = "v" + volume

        if (publisher and series and volume) is None:
            print(
                "Missing metadata from comic and will be unable to sort."
                + f"Publisher: {publisher}\nSeries: {series}\nVolume: {volume}"
            )
            return False

        new_path = pathlib.Path(self.sort_directory) / publisher / series / volume

        if not new_path.is_dir():
            try:
                new_path.mkdir(parents=True)
            except PermissionError:
                print(
                    f"due to permission error, failed to create directory: {new_path}"
                )
                return False

        original_path = pathlib.Path(comic)
        try:
            # Until python 3.9 is released, we need to force the Path
            # objects to strings so shutils.move will work correctly.
            move(fspath(original_path), fspath(new_path))
            print(f"moved '{original_path.name}' to '{new_path}'")
        except Error as e:
            print(f"Unable to move comic. Error: {e}")
            return False

        return True



"""Some miscellaneous functions"""
from pathlib import Path
from typing import Dict, List, Optional
from urllib.parse import quote_plus

from darkseid.filenameparser import FileNameParser


def cleanup_string(path_name: Optional[str]) -> Optional[str]:
    """Function to remove some characters that don't play nicely on Windows machines filesystem"""
    if path_name is not None:
        path_name = path_name.replace("/", "-")
        path_name = path_name.replace(" :", " -")
        path_name = path_name.replace(": ", " - ")
        path_name = path_name.replace(":", "-")
        path_name = path_name.replace("?", "")
    else:
        return None

    return path_name


def create_issue_query_dict(filename: Path) -> Dict[str, str]:
    """Function to create a diction of values based on the provided filename"""
    fnp = FileNameParser()
    fnp.parse_filename(filename)

    # Substitute colon for hyphen when searching for series name
    fixed_txt: str = fnp.series.replace(" - ", ": ")
    series_word_list: List[str] = fixed_txt.split()
    series_string: str = " ".join(series_word_list).strip()
    series_string = quote_plus(series_string.encode("utf-8"))
    # If there isn't an issue number, let's assume it's "1".
    number: str = quote_plus(fnp.issue.encode("utf-8")) if fnp.issue else "1"
    return {
        "series": series_string,
        "volume": fnp.volume,
        "number": number,
        "year": fnp.year,
    }



"""Functions for renaming files based on metadata"""

# Copyright 2012-2014 Anthony Beville
# Copyright 2020 Brian Pepple

import datetime
import re
from pathlib import Path
from typing import Optional

from darkseid.genericmetadata import GenericMetadata
from darkseid.issuestring import IssueString
from darkseid.utils import unique_file

from metrontagger.taggerlib.utils import cleanup_string


class FileRenamer:
    """Class to rename a comic archive based on it's metadata tag"""

    def __init__(self, metadata: GenericMetadata) -> None:
        self.set_metadata(metadata)
        self.set_template("%series% v%volume% #%issue% (of %issuecount%) (%year%)")
        self.smart_cleanup = True
        self.issue_zero_padding = 3

    def set_metadata(self, metadata: GenericMetadata) -> None:
        """Method to set the metadata"""
        self.metdata = metadata

    def set_issue_zero_padding(self, count: int) -> None:
        """Method to set the padding for the issue's number"""
        self.issue_zero_padding = count

    def set_template(self, template: str) -> None:
        """
        Method to use a user's custom file naming template.
        Currently this hasn't been implemented
        """
        self.template = template

    def replace_token(self, text: str, value: Optional[str], token: str) -> str:
        """Method to replace a value with another value"""
        # helper func
        def is_token(word: str) -> bool:
            return word[0] == "%" and word[-1:] == "%"

        if value is not None:
            return text.replace(token, str(value))
        else:
            if self.smart_cleanup:
                # smart cleanup means we want to remove anything appended to token if it's empty
                # (e.g "#%issue%"  or "v%volume%")
                # (TODO: This could fail if there is more than one token appended together, I guess)
                text_list = text.split()

                # special case for issuecount, remove preceding non-token word,
                # as in "...(of %issuecount%)..."
                if token == "%issuecount%":
                    for idx, word in enumerate(text_list):
                        if token in word and not is_token(text_list[idx - 1]):
                            text_list[idx - 1] = ""

                text_list = [x for x in text_list if token not in x]
                return " ".join(text_list)
            else:
                return text.replace(token, "")

    def smart_cleanup_string(self, new_name: str) -> str:
        # remove empty braces,brackets, parentheses
        new_name = re.sub(r"\(\s*[-:]*\s*\)", "", new_name)
        new_name = re.sub(r"\[\s*[-:]*\s*\]", "", new_name)
        new_name = re.sub(r"\{\s*[-:]*\s*\}", "", new_name)

        # remove duplicate spaces
        new_name = " ".join(new_name.split())

        # remove remove duplicate -, _,
        new_name = re.sub(r"[-_]{2,}\s+", "-- ", new_name)
        new_name = re.sub(r"(\s--)+", " --", new_name)
        new_name = re.sub(r"(\s-)+", " -", new_name)

        # remove dash or double dash at end of line
        new_name = re.sub(r"[-]{1,2}\s*$", "", new_name)

        # remove duplicate spaces (again!)
        new_name = " ".join(new_name.split())

        return new_name

    def determine_name(
        self, filename: Path, ext: Optional[str] = None
    ) -> Optional[str]:
        """Method to create the new filename based on the files metadata"""
        meta_data = self.metdata
        new_name = self.template

        new_name = self.replace_token(new_name, meta_data.series, "%series%")
        new_name = self.replace_token(new_name, meta_data.volume, "%volume%")

        if meta_data.issue is not None:
            issue_str = "{0}".format(
                IssueString(meta_data.issue).as_string(pad=self.issue_zero_padding)
            )
        else:
            issue_str = None
        new_name = self.replace_token(new_name, issue_str, "%issue%")

        new_name = self.replace_token(new_name, meta_data.issue_count, "%issuecount%")
        new_name = self.replace_token(new_name, meta_data.year, "%year%")
        new_name = self.replace_token(new_name, meta_data.publisher, "%publisher%")
        new_name = self.replace_token(new_name, meta_data.title, "%title%")
        new_name = self.replace_token(new_name, meta_data.month, "%month%")
        month_name = None
        if (
            meta_data.month is not None
            and (
                (isinstance(meta_data.month, str) and meta_data.month.isdigit())
                or isinstance(meta_data.month, int)
            )
            and int(meta_data.month) in range(1, 13)
        ):
            date_time = datetime.datetime(1970, int(meta_data.month), 1, 0, 0)
            month_name = date_time.strftime("%B")
        new_name = self.replace_token(new_name, month_name, "%month_name%")

        new_name = self.replace_token(new_name, meta_data.genre, "%genre%")
        new_name = self.replace_token(new_name, meta_data.language, "%language_code%")
        new_name = self.replace_token(
            new_name, meta_data.critical_rating, "%criticalrating%"
        )
        new_name = self.replace_token(
            new_name, meta_data.alternate_series, "%alternateseries%"
        )
        new_name = self.replace_token(
            new_name, meta_data.alternate_number, "%alternatenumber%"
        )
        new_name = self.replace_token(
            new_name, meta_data.alternate_count, "%alternatecount%"
        )
        new_name = self.replace_token(new_name, meta_data.imprint, "%imprint%")
        new_name = self.replace_token(new_name, meta_data.format, "%format%")
        new_name = self.replace_token(
            new_name, meta_data.maturity_rating, "%maturityrating%"
        )
        new_name = self.replace_token(new_name, meta_data.story_arc, "%storyarc%")
        new_name = self.replace_token(new_name, meta_data.series_group, "%seriesgroup%")
        new_name = self.replace_token(new_name, meta_data.scan_info, "%scaninfo%")

        if self.smart_cleanup:
            new_name = self.smart_cleanup_string(new_name)

        if ext is None:
            ext = filename.suffix

        new_name += ext

        # some tweaks to keep various filesystems happy
        new_name = cleanup_string(new_name)

        return new_name

    def rename_file(self, comic: Path) -> Optional[Path]:
        new_name = self.determine_name(comic)

        if new_name == comic.name:
            print("Filename is already good!")
            return None

        unique_name = unique_file(comic.parent / new_name)
        comic.rename(unique_name)

        return unique_name



"""Setup file for metron-tagger"""
from setuptools import find_packages, setup

import metrontagger

setup(
    name="metron-tagger",
    version=metrontagger.VERSION,
    description="A program to write metadata from metron.cloud to a comic archive",
    long_description=open("README.rst").read(),
    long_description_content_type="text/x-rst",
    author="Brian Pepple",
    author_email="bdpepple@gmail.com",
    url="https://github.com/bpepple/metron-tagger",
    license="GPLv3",
    packages=find_packages(exclude=["tests"]),
    python_requires=">=3.8",
    install_requires=["darkseid >= 1.0.6", "ratelimit"],
    entry_points={"console_scripts": ["metron-tagger=metrontagger.main:main"]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
        "Topic :: Other/Nonlisted Topic",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
    ],
    keywords=["comics", "comic", "metadata", "tagging", "tagger"],
)
